@@ -13,54 +13,54 @@
     SPDX-License-Identifier: GPL-2.0-or-later
  */
 
+#include <config-libkleo.h>
+
 #include "keyrequester.h"
 
 #include "keyselectiondialog.h"
 
-#include "libkleo/dn.h"
+#include <libkleo/algorithm.h>
+#include <libkleo/compliance.h>
+#include <libkleo/dn.h>
+#include <libkleo/formatting.h>
 
-// gpgme++
-#include <gpgme++/key.h>
-#include <gpgme++/keylistresult.h>
-#include <qgpgme/keylistjob.h>
-
-// KDE
 #include <KLocalizedString>
-#include <QDialog>
 #include <KMessageBox>
-#include <QPushButton>
 
-// Qt
+#include <QGpgME/KeyListJob>
+
 #include <QApplication>
-
+#include <QDialog>
+#include <QHBoxLayout>
+#include <QPushButton>
 #include <QString>
 
-#include <QHBoxLayout>
-
+#include <gpgme++/key.h>
+#include <gpgme++/keylistresult.h>
 
 using namespace QGpgME;
+using namespace Kleo;
 
-Kleo::KeyRequester::KeyRequester(unsigned int allowedKeys, bool multipleKeys,
-                                 QWidget *parent)
-    : QWidget(parent),
-      mOpenPGPBackend(nullptr),
-      mSMIMEBackend(nullptr),
-      mMulti(multipleKeys),
-      mKeyUsage(allowedKeys),
-      mJobs(0),
-      d(nullptr)
+Kleo::KeyRequester::KeyRequester(unsigned int allowedKeys, bool multipleKeys, QWidget *parent)
+    : QWidget(parent)
+    , mOpenPGPBackend(nullptr)
+    , mSMIMEBackend(nullptr)
+    , mMulti(multipleKeys)
+    , mKeyUsage(allowedKeys)
+    , mJobs(0)
+    , d(nullptr)
 {
     init();
 }
 
 Kleo::KeyRequester::KeyRequester(QWidget *parent)
-    : QWidget(parent),
-      mOpenPGPBackend(nullptr),
-      mSMIMEBackend(nullptr),
-      mMulti(false),
-      mKeyUsage(0),
-      mJobs(0),
-      d(nullptr)
+    : QWidget(parent)
+    , mOpenPGPBackend(nullptr)
+    , mSMIMEBackend(nullptr)
+    , mMulti(false)
+    , mKeyUsage(0)
+    , mJobs(0)
+    , d(nullptr)
 {
     init();
 }
@@ -70,6 +70,11 @@ void Kleo::KeyRequester::init()
     auto hlay = new QHBoxLayout(this);
     hlay->setContentsMargins(0, 0, 0, 0);
 
+    if (DeVSCompliance::isCompliant()) {
+        mComplianceIcon = new QLabel{this};
+        mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-question")).pixmap(22));
+    }
+
     // the label where the key id is to be displayed:
     mLabel = new QLabel(this);
     mLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
@@ -77,15 +82,18 @@ void Kleo::KeyRequester::init()
     // the button to unset any key:
     mEraseButton = new QPushButton(this);
     mEraseButton->setAutoDefault(false);
-    mEraseButton->setSizePolicy(QSizePolicy(QSizePolicy::Minimum,
-                                            QSizePolicy::Minimum));
-    mEraseButton->setIcon(QIcon::fromTheme(QApplication::isRightToLeft() ? QStringLiteral("edit-clear-locationbar-ltr") : QStringLiteral("edit-clear-locationbar-rtl")));
+    mEraseButton->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
+    mEraseButton->setIcon(
+        QIcon::fromTheme(QApplication::isRightToLeft() ? QStringLiteral("edit-clear-locationbar-ltr") : QStringLiteral("edit-clear-locationbar-rtl")));
     mEraseButton->setToolTip(i18n("Clear"));
 
     // the button to call the KeySelectionDialog:
     mDialogButton = new QPushButton(i18n("Change..."), this);
     mDialogButton->setAutoDefault(false);
 
+    if (mComplianceIcon) {
+        hlay->addWidget(mComplianceIcon);
+    }
     hlay->addWidget(mLabel, 1);
     hlay->addWidget(mEraseButton);
     hlay->addWidget(mDialogButton);
@@ -93,15 +101,13 @@ void Kleo::KeyRequester::init()
     connect(mEraseButton, &QPushButton::clicked, this, &SigningKeyRequester::slotEraseButtonClicked);
     connect(mDialogButton, &QPushButton::clicked, this, &SigningKeyRequester::slotDialogButtonClicked);
 
-    setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
-                              QSizePolicy::Fixed));
+    setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed));
 
     setAllowedKeys(mKeyUsage);
 }
 
 Kleo::KeyRequester::~KeyRequester()
 {
-
 }
 
 const std::vector<GpgME::Key> &Kleo::KeyRequester::keys() const
@@ -122,10 +128,11 @@ const GpgME::Key &Kleo::KeyRequester::key() const
 void Kleo::KeyRequester::setKeys(const std::vector<GpgME::Key> &keys)
 {
     mKeys.clear();
-    for (auto it = keys.begin(); it != keys.end(); ++it)
+    for (auto it = keys.begin(); it != keys.end(); ++it) {
         if (!it->isNull()) {
             mKeys.push_back(*it);
         }
+    }
     updateKeys();
 }
 
@@ -150,11 +157,13 @@ QString Kleo::KeyRequester::fingerprint() const
 QStringList Kleo::KeyRequester::fingerprints() const
 {
     QStringList result;
-    for (auto it = mKeys.begin(); it != mKeys.end(); ++it)
-        if (!it->isNull())
+    for (auto it = mKeys.begin(); it != mKeys.end(); ++it) {
+        if (!it->isNull()) {
             if (const char *fpr = it->primaryFingerprint()) {
                 result.push_back(QLatin1String(fpr));
             }
+        }
+    }
     return result;
 }
 
@@ -168,9 +177,20 @@ void Kleo::KeyRequester::setFingerprints(const QStringList &fingerprints)
     startKeyListJob(fingerprints);
 }
 
+static bool keyIsCompliant(const GpgME::Key &key)
+{
+    return (key.keyListMode() & GpgME::Validate) //
+        && Formatting::uidsHaveFullValidity(key) //
+        && Formatting::isKeyDeVs(key);
+}
+
 void Kleo::KeyRequester::updateKeys()
 {
     if (mKeys.empty()) {
+        if (mComplianceIcon) {
+            mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-unavailable")).pixmap(22));
+            mComplianceIcon->setToolTip(QString{});
+        }
         mLabel->clear();
         return;
     }
@@ -187,18 +207,26 @@ void Kleo::KeyRequester::updateKeys()
         const QString fpr = QLatin1String(it->primaryFingerprint());
         labelTexts.push_back(fpr.right(8));
         toolTipText += fpr.right(8) + QLatin1String(": ");
-        if (const char *uid = it->userID(0).id())
+        if (const char *uid = it->userID(0).id()) {
             if (it->protocol() == GpgME::OpenPGP) {
                 toolTipText += QString::fromUtf8(uid);
             } else {
                 toolTipText += Kleo::DN(uid).prettyDN();
             }
-        else {
+        } else {
             toolTipText += xi18n("<placeholder>unknown</placeholder>");
         }
         toolTipText += QLatin1Char('\n');
     }
-
+    if (mComplianceIcon) {
+        if (Kleo::all_of(mKeys, &keyIsCompliant)) {
+            mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-success")).pixmap(22));
+            mComplianceIcon->setToolTip(DeVSCompliance::name(true));
+        } else {
+            mComplianceIcon->setPixmap(QIcon::fromTheme(QStringLiteral("emblem-information")).pixmap(22));
+            mComplianceIcon->setToolTip(DeVSCompliance::name(false));
+        }
+    }
     mLabel->setText(labelTexts.join(QLatin1String(", ")));
     mLabel->setToolTip(toolTipText);
 }
@@ -208,10 +236,11 @@ void Kleo::KeyRequester::updateKeys()
 static void showKeyListError(QWidget *parent, const GpgME::Error &err)
 {
     Q_ASSERT(err);
-    const QString msg = i18n("<qt><p>An error occurred while fetching "
-                             "the keys from the backend:</p>"
-                             "<p><b>%1</b></p></qt>",
-                             QString::fromLocal8Bit(err.asString()));
+    const QString msg = i18n(
+        "<qt><p>An error occurred while fetching "
+        "the keys from the backend:</p>"
+        "<p><b>%1</b></p></qt>",
+        QString::fromLocal8Bit(err.asString()));
 
     KMessageBox::error(parent, msg, i18n("Key Listing Failed"));
 }
@@ -227,10 +256,11 @@ void Kleo::KeyRequester::startKeyListJob(const QStringList &fingerprints)
     mJobs = 0;
 
     unsigned int count = 0;
-    for (QStringList::const_iterator it = fingerprints.begin(); it != fingerprints.end(); ++it)
+    for (QStringList::const_iterator it = fingerprints.begin(); it != fingerprints.end(); ++it) {
         if (!(*it).trimmed().isEmpty()) {
             ++count;
         }
+    }
 
     if (!count) {
         // don't fall into the trap that an empty pattern means
@@ -240,7 +270,7 @@ void Kleo::KeyRequester::startKeyListJob(const QStringList &fingerprints)
     }
 
     if (mOpenPGPBackend) {
-        KeyListJob *job = mOpenPGPBackend->keyListJob(false);    // local, no sigs
+        KeyListJob *job = mOpenPGPBackend->keyListJob(false); // local, no sigs
         if (!job) {
             KMessageBox::error(this,
                                i18n("The OpenPGP backend does not support listing keys. "
@@ -250,9 +280,8 @@ void Kleo::KeyRequester::startKeyListJob(const QStringList &fingerprints)
             connect(job, &KeyListJob::result, this, &SigningKeyRequester::slotKeyListResult);
             connect(job, &KeyListJob::nextKey, this, &SigningKeyRequester::slotNextKey);
 
-            const GpgME::Error err = job->start(fingerprints,
-                                                mKeyUsage & Kleo::KeySelectionDialog::SecretKeys &&
-                                                !(mKeyUsage & Kleo::KeySelectionDialog::PublicKeys));
+            const GpgME::Error err =
+                job->start(fingerprints, mKeyUsage & Kleo::KeySelectionDialog::SecretKeys && !(mKeyUsage & Kleo::KeySelectionDialog::PublicKeys));
 
             if (err) {
                 showKeyListError(this, err);
@@ -263,7 +292,7 @@ void Kleo::KeyRequester::startKeyListJob(const QStringList &fingerprints)
     }
 
     if (mSMIMEBackend) {
-        KeyListJob *job = mSMIMEBackend->keyListJob(false);    // local, no sigs
+        KeyListJob *job = mSMIMEBackend->keyListJob(false); // local, no sigs
         if (!job) {
             KMessageBox::error(this,
                                i18n("The S/MIME backend does not support listing keys. "
@@ -273,9 +302,8 @@ void Kleo::KeyRequester::startKeyListJob(const QStringList &fingerprints)
             connect(job, &KeyListJob::result, this, &SigningKeyRequester::slotKeyListResult);
             connect(job, &KeyListJob::nextKey, this, &SigningKeyRequester::slotNextKey);
 
-            const GpgME::Error err = job->start(fingerprints,
-                                                mKeyUsage & Kleo::KeySelectionDialog::SecretKeys &&
-                                                !(mKeyUsage & Kleo::KeySelectionDialog::PublicKeys));
+            const GpgME::Error err =
+                job->start(fingerprints, mKeyUsage & Kleo::KeySelectionDialog::SecretKeys && !(mKeyUsage & Kleo::KeySelectionDialog::PublicKeys));
 
             if (err) {
                 showKeyListError(this, err);
@@ -315,9 +343,8 @@ void Kleo::KeyRequester::slotKeyListResult(const GpgME::KeyListResult &res)
 
 void Kleo::KeyRequester::slotDialogButtonClicked()
 {
-    KeySelectionDialog *dlg = mKeys.empty()
-                              ? new KeySelectionDialog(mDialogCaption, mDialogMessage, mInitialQuery, mKeyUsage, mMulti, false, this)
-                              : new KeySelectionDialog(mDialogCaption, mDialogCaption, mKeys, mKeyUsage, mMulti, false, this);
+    KeySelectionDialog *dlg = mKeys.empty() ? new KeySelectionDialog(mDialogCaption, mDialogMessage, mInitialQuery, mKeyUsage, mMulti, false, this)
+                                            : new KeySelectionDialog(mDialogCaption, mDialogCaption, mKeys, mKeyUsage, mMulti, false, this);
 
     if (dlg->exec() == QDialog::Accepted) {
         if (mMulti) {
@@ -437,47 +464,51 @@ static inline unsigned int signingKeyUsage(bool openpgp, bool smime, bool truste
     return foo(openpgp, smime, trusted, valid) | Kleo::KeySelectionDialog::SigningKeys | Kleo::KeySelectionDialog::SecretKeys;
 }
 
-Kleo::EncryptionKeyRequester::EncryptionKeyRequester(bool multi, unsigned int proto,
-        QWidget *parent,
-        bool onlyTrusted, bool onlyValid)
-    : KeyRequester(encryptionKeyUsage(proto & OpenPGP, proto & SMIME, onlyTrusted, onlyValid), multi,
-                   parent), d(nullptr)
+Kleo::EncryptionKeyRequester::EncryptionKeyRequester(bool multi, unsigned int proto, QWidget *parent, bool onlyTrusted, bool onlyValid)
+    : KeyRequester(encryptionKeyUsage(proto & OpenPGP, proto & SMIME, onlyTrusted, onlyValid), multi, parent)
+    , d(nullptr)
 {
 }
 
 Kleo::EncryptionKeyRequester::EncryptionKeyRequester(QWidget *parent)
-    : KeyRequester(0, false, parent), d(nullptr)
+    : KeyRequester(0, false, parent)
+    , d(nullptr)
 {
 }
 
-Kleo::EncryptionKeyRequester::~EncryptionKeyRequester() {}
+Kleo::EncryptionKeyRequester::~EncryptionKeyRequester()
+{
+}
 
 void Kleo::EncryptionKeyRequester::setAllowedKeys(unsigned int proto, bool onlyTrusted, bool onlyValid)
 {
     KeyRequester::setAllowedKeys(encryptionKeyUsage(proto & OpenPGP, proto & SMIME, onlyTrusted, onlyValid));
 }
 
-Kleo::SigningKeyRequester::SigningKeyRequester(bool multi, unsigned int proto,
-        QWidget *parent,
-        bool onlyTrusted, bool onlyValid)
-    : KeyRequester(signingKeyUsage(proto & OpenPGP, proto & SMIME, onlyTrusted, onlyValid), multi,
-                   parent), d(nullptr)
+Kleo::SigningKeyRequester::SigningKeyRequester(bool multi, unsigned int proto, QWidget *parent, bool onlyTrusted, bool onlyValid)
+    : KeyRequester(signingKeyUsage(proto & OpenPGP, proto & SMIME, onlyTrusted, onlyValid), multi, parent)
+    , d(nullptr)
 {
 }
 
 Kleo::SigningKeyRequester::SigningKeyRequester(QWidget *parent)
-    : KeyRequester(0, false, parent), d(nullptr)
+    : KeyRequester(0, false, parent)
+    , d(nullptr)
 {
 }
 
-Kleo::SigningKeyRequester::~SigningKeyRequester() {}
+Kleo::SigningKeyRequester::~SigningKeyRequester()
+{
+}
 
 void Kleo::SigningKeyRequester::setAllowedKeys(unsigned int proto, bool onlyTrusted, bool onlyValid)
 {
     KeyRequester::setAllowedKeys(signingKeyUsage(proto & OpenPGP, proto & SMIME, onlyTrusted, onlyValid));
 }
 
-void Kleo::KeyRequester::virtual_hook(int, void *) {}
+void Kleo::KeyRequester::virtual_hook(int, void *)
+{
+}
 void Kleo::EncryptionKeyRequester::virtual_hook(int id, void *data)
 {
     KeyRequester::virtual_hook(id, data);
@@ -486,4 +517,3 @@ void Kleo::SigningKeyRequester::virtual_hook(int id, void *data)
 {
     KeyRequester::virtual_hook(id, data);
 }
-
