@@ -19,6 +19,7 @@
 
 #include "assuan.h"
 #include "compat.h"
+#include "compliance.h"
 #include "cryptoconfig.h"
 #include "hex.h"
 
@@ -62,6 +63,12 @@ QString Kleo::gnupgHomeDirectory()
     return homeDir;
 }
 
+QString Kleo::gnupgPrivateKeysDirectory()
+{
+    static const QString dir = QDir{gnupgHomeDirectory()}.filePath(QStringLiteral("private-keys-v1.d"));
+    return dir;
+}
+
 int Kleo::makeGnuPGError(int code)
 {
     return gpg_error(static_cast<gpg_err_code_t>(code));
@@ -125,7 +132,7 @@ QStringList Kleo::gnupgFolderWhitelist()
     static const QDir gnupgHome{gnupgHomeDirectory()};
     return {
         gnupgHome.path(),
-        gnupgHome.filePath(QStringLiteral("private-keys-v1.d")),
+        gnupgPrivateKeysDirectory(),
     };
 }
 
@@ -148,6 +155,8 @@ private:
     QString mVersion;
     QString mDescription;
     QString mDescLong;
+    QString mBrandingWindowTitle;
+    QString mBrandingIcon;
     bool mSignedVersion;
 
     Gpg4win()
@@ -160,21 +169,18 @@ private:
         const QString verPath = instPath + QStringLiteral("/../VERSION");
         QFile versionFile(verPath);
 
-        QString versVersion;
-        QString versDescription;
-        QString versDescLong;
         // Open the file first to avoid a verify and then read issue where
         // "auditors" might say its an issue,...
         if (!versionFile.open(QIODevice::ReadOnly)) {
-            // No need to translate this should only be the case in development
-            // builds.
             return;
-        } else {
-            // Expect a three line format of three HTML strings.
-            versVersion = QString::fromUtf8(versionFile.readLine()).trimmed();
-            versDescription = QString::fromUtf8(versionFile.readLine()).trimmed();
-            versDescLong = QString::fromUtf8(versionFile.readLine()).trimmed();
         }
+        // Expect a three line format of three HTML strings.
+        const auto versVersion = QString::fromUtf8(versionFile.readLine()).trimmed();
+        const auto versDescription = QString::fromUtf8(versionFile.readLine()).trimmed();
+        const auto versDescLong = QString::fromUtf8(versionFile.readLine()).trimmed();
+        // read optional two branding strings
+        const auto brandingWindowTitle = QString::fromUtf8(versionFile.readLine()).trimmed();
+        const auto brandingIcon = QString::fromUtf8(versionFile.readLine()).trimmed();
 
         const QString sigPath = verPath + QStringLiteral(".sig");
         QFileInfo versionSig(instPath + QStringLiteral("/../VERSION.sig"));
@@ -192,6 +198,8 @@ private:
                 mVersion = versVersion;
                 mDescription = versDescription;
                 mDescLong = versDescLong;
+                mBrandingWindowTitle = brandingWindowTitle;
+                mBrandingIcon = brandingIcon;
                 mSignedVersion = true;
             } else {
                 qCDebug(LIBKLEO_LOG) << "gpgv failed with stderr: " << gpgv.readAllStandardError();
@@ -221,6 +229,14 @@ public:
     {
         return mSignedVersion;
     }
+    const QString &brandingWindowTitle() const
+    {
+        return mBrandingWindowTitle;
+    }
+    const QString &brandingIcon() const
+    {
+        return mBrandingIcon;
+    }
 };
 } // namespace
 
@@ -261,6 +277,15 @@ QString Kleo::gpg4winLongDescription()
     return Gpg4win::instance()->longDescription();
 }
 
+QString Kleo::brandingWindowTitle()
+{
+    return Gpg4win::instance()->brandingWindowTitle();
+}
+QString Kleo::brandingIcon()
+{
+    return Gpg4win::instance()->brandingIcon();
+}
+
 QString Kleo::gpg4winInstallPath()
 {
 #ifdef Q_OS_WIN
@@ -467,24 +492,12 @@ bool Kleo::gpgComplianceP(const char *mode)
 
 bool Kleo::gnupgUsesDeVsCompliance()
 {
-    return getCryptoConfigStringValue("gpg", "compliance") == QLatin1String{"de-vs"};
+    return DeVSCompliance::isActive();
 }
 
 bool Kleo::gnupgIsDeVsCompliant()
 {
-    if (!gnupgUsesDeVsCompliance()) {
-        return false;
-    }
-    // The pseudo option compliance_de_vs was fully added in 2.2.34;
-    // For versions between 2.2.28 and 2.2.33 there was a broken config
-    // value with a wrong type. So for them we add an extra check. This
-    // can be removed in future versions because. For GnuPG we could assume
-    // non-compliance for older versions as versions of Kleopatra for
-    // which this matters are bundled with new enough versions of GnuPG anyway
-    if (engineIsVersion(2, 2, 28) && !engineIsVersion(2, 2, 34)) {
-        return true;
-    }
-    return getCryptoConfigIntValue("gpg", "compliance_de_vs", 0) != 0;
+    return DeVSCompliance::isCompliant();
 }
 
 enum GpgME::UserID::Validity Kleo::keyValidity(const GpgME::Key &key)
@@ -501,6 +514,37 @@ enum GpgME::UserID::Validity Kleo::keyValidity(const GpgME::Key &key)
 }
 
 #ifdef Q_OS_WIN
+static unsigned int gpgConfGetConsoleOutputCodePage()
+{
+    // calls `gpgconf --show-codepages` to determine the console output codepage used by gpg on Windows;
+    // for other OSs gpgconf returns nothing
+    const auto gpgConfPath = Kleo::gpgConfPath();
+    if (gpgConfPath.isEmpty()) {
+        return 0;
+    }
+    QProcess gpgConf;
+    qCDebug(LIBKLEO_LOG) << __func__ << "starting" << gpgConfPath << "--show-codepages";
+    gpgConf.start(gpgConfPath, {QStringLiteral("--show-codepages")});
+    if (!gpgConf.waitForFinished()) {
+        qCDebug(LIBKLEO_LOG) << __func__ << "failed to execute gpgconf:" << gpgConf.errorString() << "\nstderr:" << gpgConf.readAllStandardError();
+        return 0;
+    }
+
+    unsigned int cpno = 0;
+    const auto lines = gpgConf.readAllStandardOutput().split('\n');
+    // look for a line of the form "Console: CP%u" or "Console: CP%u/CP%u"
+    for (const auto &l : lines) {
+        if (l.startsWith("Console: CP")) {
+            // the console output codepage is the second CP value if there are two
+            cpno = l.mid(l.lastIndexOf("CP") + 2).toUInt();
+            break;
+        }
+    }
+
+    qCDebug(LIBKLEO_LOG) << __func__ << "returns" << cpno;
+    return cpno;
+}
+
 static QString fromEncoding(unsigned int src_encoding, const char *data)
 {
     int n = MultiByteToWideChar(src_encoding, 0, data, -1, NULL, 0);
@@ -524,31 +568,18 @@ static QString fromEncoding(unsigned int src_encoding, const char *data)
 QString Kleo::stringFromGpgOutput(const QByteArray &ba)
 {
 #ifdef Q_OS_WIN
-    /* Qt on Windows uses GetACP while GnuPG prefers
-     * GetConsoleOutputCP.
-     *
-     * As we are not a console application GetConsoleOutputCP
-     * usually returns 0.
-     * From experience the closest thing that let's us guess
-     * what GetConsoleOutputCP returns for a console application
-     * it appears to be the OEMCP.
-     */
-    unsigned int cpno = GetConsoleOutputCP();
-    if (!cpno) {
-        cpno = GetOEMCP();
-    }
-    if (!cpno) {
-        cpno = GetACP();
-    }
-    if (!cpno) {
-        qCDebug(LIBKLEO_LOG) << "Failed to find native codepage";
-        return QString();
-    }
+    static const unsigned int cpno = gpgConfGetConsoleOutputCodePage();
 
-    return fromEncoding(cpno, ba.constData());
-#else
-    return QString::fromLocal8Bit(ba);
+    if (cpno) {
+        qCDebug(LIBKLEO_LOG) << __func__ << "trying to decode" << ba << "using codepage" << cpno;
+        const auto s = fromEncoding(cpno, ba.constData());
+        if (!s.isEmpty() || ba.isEmpty()) {
+            return s;
+        }
+        qCDebug(LIBKLEO_LOG) << __func__ << "decoding output failed; falling back to QString::fromLocal8Bit()";
+    }
 #endif
+    return QString::fromLocal8Bit(ba);
 }
 
 QStringList Kleo::backendVersionInfo()
